package hamster;

/**
 * 
 * Der Hamster soll einen Berg erklimmen. Voraussetzung: Er steht mit
 * Blickrichtung OST vor diesem Berg, dessen Stufen jeweils eine Kachel hoch
 * sind.
 * 
 * @author Dietrich Boles (Universitaet Oldenburg)
 * @version 1.0 (25.01.2006)
 * 
 */
public class BergsteigerHamster extends Hamster {

	/**
	 * initialisiert einen neuen BergsteigerHamster mit den uebergebenen Werten
	 * 
	 * @param reihe
	 *            Reihe der Kachel
	 * @param spalte
	 *            Spalte der Kachel
	 * @param blickrichtung
	 *            Blickrichtung des Hamsters
	 * @param anzahlKoerner
	 *            Anzahl an Koernern im Maul des Hamsters
	 */
	public BergsteigerHamster(int reihe, int spalte, int blickrichtung,
			int anzahlKoerner) {
		super(reihe, spalte, blickrichtung, anzahlKoerner);
	}

	/**
	 * laedt ein vordefiniertes Territorium und erzeugt und initialisiert darin
	 * einen neuen BergsteigerHamster mit den uebergebenen Werten
	 * 
	 * @param reihe
	 *            Reihe der Kachel
	 * @param spalte
	 *            Spalte der Kachel
	 * @param blickrichtung
	 *            Blickrichtung des Hamsters
	 * @param anzahlKoerner
	 *            Anzahl an Koernern im Maul des Hamsters
	 * @param territoriumsDatei
	 *            Name einer Datei mit einem zu ladenden Territorium
	 */
	public BergsteigerHamster(int reihe, int spalte, int blickrichtung,
			int anzahlKoerner, String territoriumsDatei) {
		super();
		Territorium.ladeTerritorium(territoriumsDatei);
		this.init(reihe, spalte, blickrichtung, anzahlKoerner);
	}

	/**
	 * laedt ein vordefiniertes Territorium und initialisiert einen neuen
	 * BergsteigerHamster mit den Attributwerten eines bereits existierenden
	 * Hamsters
	 * 
	 * @param existierenderHamster
	 *            existierender Hamster
	 * @param territoriumsDatei
	 *            Name einer Datei mit einem zu ladenden Territorium
	 */
	public BergsteigerHamster(Hamster existierenderHamster,
			String territoriumsDatei) {
		super();
		Territorium.ladeTerritorium(territoriumsDatei);
		this.init(existierenderHamster.getReihe(), existierenderHamster
				.getSpalte(), existierenderHamster.getBlickrichtung(),
				existierenderHamster.getAnzahlKoerner());
	}

	/**
	 * Der Hamster erklimmt den Berg. Voraussetzung: Er steht mit Blickrichtung
	 * OST vor einem Berg, dessen Stufen jeweils eine Kachel hoch sind.
	 */
	public void erklimmeGifel() {
		laufeZumBerg();
		erklimmeGipfel();
	}

	void laufeZumBerg() {
		while (vornFrei()) {
			vor();
		}
	}

	void erklimmeGipfel() {
		do {
			erklimmeEineStufe();
		} while (!vornFrei());
	}

	void erklimmeEineStufe() {
		linksUm();
		vor();
		rechtsUm();
		vor();
	}

	void rechtsUm() {
		kehrt();
		linksUm();
	}

	void kehrt() {
		linksUm();
		linksUm();
	}
}